package za.co.momentum.recurring.client.thread;

import za.co.momentum.recurring.client.exceptions.TimeoutException;
import za.co.momentum.recurring.client.thread.services.ServiceThreadIFace;

/**
 * This class is the actual service thread which will run the service and time
 * out if the elapsed time has expired.
 * 
 * @author Deon van Aarde 
 * Momentum Risk & Savings Created on 2005/08/30
 * 
 */
public class ServiceThreadListner implements ThreadListenerIFace {
	
	/*
	 * Indicate if the action is complete.
	 */
	private boolean actionComplete = false;

	/*
	 * This is the resulting object for a service action being executed.
	 */
	private Object returnObject = null;
	
	
	/*
	 * This is the timeout setting of the thread
	 */
	private long timeoutInSeconds = 10;

	/*
	 * This is the actual service thread interface.
	 */
	private ServiceThreadIFace thread = null;
	
	/*
	 * Use this variable to see if the service timed out
	 */
	private boolean timedOut= false;
	
	private long startTime;

	private ServiceThreadListner() {
	}

	public ServiceThreadListner(ServiceThreadIFace thread) {
		this.thread = thread;
	}

	public void threadResult(ThreadEvent threadResult) {
		returnObject = threadResult.getResult();
		actionComplete = true;
	}

	public void threadExceptionThrown(ThreadEvent problem) {
		returnObject = problem.getException();
		actionComplete = true;
	}

	public Object getReturnObject() {
		return returnObject;
	}

	public void runService(long timeOutSeconds) {
	//	try {
			actionComplete = false;
			timedOut = false;
			returnObject = null;
			this.timeoutInSeconds = timeOutSeconds;
			startTime = System.currentTimeMillis();
			thread.addThreadListener(this);
			thread.start();
			//while (!actionComplete) {
			//	checkTimer(startTime, timeOutSeconds);
			//}
	//	} finally {
	//		thread.removeThreadListener(this);
	//	}
	}


	/**
	 * @param startTime
	 */
	public void checkTimer() {
		//Logger.getLogger(ServiceThreadListner.class).debug("In check timer");
		if (((1000 * timeoutInSeconds) + startTime) < System.currentTimeMillis()) {
			timedOut = true;
			returnObject = null;
			throw new TimeoutException("TIMEOUT of " + timeoutInSeconds
					+ " seconds");
		}
	}

	public boolean isActionComplete() {
		return actionComplete;
	}

	public void setActionComplete(boolean actionComplete) {
		this.actionComplete = actionComplete;
	}

	public long getTimeout() {
		return timeoutInSeconds;
	}

	public boolean isTimedOut() {
		return timedOut;
	}

}