package za.co.momentum.recurring.nurse.client.reports;

import java.io.IOException;
import java.net.URL;

import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFCellStyle;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.eclipse.core.runtime.Platform;
import za.co.momentum.recurring.nurse.util.TNLogger;

public class ContractedNursesReport extends ExcelReport {
	
	private double total = 0;
	
	public ContractedNursesReport(ExcelData data, String filename) {
		super(data, filename);
		try {
			URL furl = Platform.getProduct().getDefiningBundle().getEntry(
			"contractedNursesTemplate.xls");
			templateName = Platform.asLocalURL(furl).getPath();
			TNLogger.debug("Template path = " + templateName);
		} catch (IOException e) {
			TNLogger.error("Could not find template contractedNursesTemplate.xls "
					+ e.getMessage());
		}
		//templateName = "Contracted Nurses report";
	}

	protected void writeReport() {
		HSSFCell c = getCell("$date");
		c.setCellValue(data.getTheDate());
		c = getCell("$startdate");
		c.setCellValue(data.getStartDate());
		c = getCell("$enddate");
		c.setCellValue(data.getEndDate());
		c = getCell("$rate");
		c.setCellValue(data.getContractRate());
		c = getCell("$nurses");
		insertNurses(c);
		c = getCell("$totalDue");
		c.setCellValue("R" + total);
		TNLogger.debug("Finished writing report ContractedNursesReport");
	}

	private void insertNurses(HSSFCell startCell) {
		HSSFCell temp = startCell;
		HSSFRow row = sheet.getRow(rownum);
		NurseReportDetail[] detail = data.getNurseReportDetail();
		for (int i = 0; i < detail.length; i++) {
			temp = row.createCell((short) 0);
			temp.setCellValue(detail[i].getName());
			temp = setAlignment(temp, HSSFCellStyle.ALIGN_LEFT);
			temp = row.createCell((short) (temp.getCellNum() + 1));
			temp.setCellValue(detail[i].getEmployeeNumber());
			temp = setAlignment(temp, HSSFCellStyle.ALIGN_LEFT);
			temp = row.createCell((short) (temp.getCellNum() + 2));
			temp.setCellValue(detail[i].getNumberOfAppointmentDone());
			temp = setAlignment(temp, HSSFCellStyle.ALIGN_CENTER);
			temp = row.createCell((short) (temp.getCellNum() + 3));
			temp.setCellValue("R " + detail[i].getSalary());
			temp = setAlignment(temp, HSSFCellStyle.ALIGN_RIGHT);
			total += detail[i].getSalary();
			row = sheet.createRow(row.getRowNum() + 1);
		}
		row = sheet.createRow(row.getRowNum() + 1);
		temp = row.createCell((short)0);
		temp.setCellValue("Total");
		temp = setAlignment(temp, HSSFCellStyle.ALIGN_LEFT);
		temp = row.createCell((short)6);
		temp.setCellValue("$totalDue");
		temp = setAlignment(temp, HSSFCellStyle.ALIGN_RIGHT);
	}
}
