package za.co.momentum.recurring.nurse.client.reports;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import za.co.momentum.recurring.base.Util;
import za.co.momentum.risk.fastlane.nurse.dto.AppDetailTO;
import za.co.momentum.risk.fastlane.nurse.dto.ContactDetailsTO;
import za.co.momentum.risk.fastlane.nurse.dto.PersonTO;
import za.co.momentum.recurring.nurse.util.AppReqDetailAppointmentDateComparator;

public class ExcelData {
  private String theDate;

  private String startDate;

  private String endDate;

  private String contractRate;

  private List nurses;

  private int threshold;

  public ExcelData() {
    nurses = new ArrayList();
  }

  public ExcelData(AppDetailTO[] ato) {
    nurses = new ArrayList();
    Arrays.sort(ato, new AppReqDetailAppointmentDateComparator());
    for (int i = 0; i < ato.length; i++) {
      AppDetailTO detail = ato[i];
      PersonTO per = detail.getContactPerson();
      if (per == null) {
        per = new PersonTO();
        per.setTitle("");
        per.setFirstName("Unknown");
        per.setSurname("");
      }
      ContactDetailsTO con = per.getContactDetails();
      if (con == null) {
        con = new ContactDetailsTO();
        con.setCellNo("Unknown");
        con.setWorkPhoneNo("");
        con.setHomePhoneNo("");
      }

      NurseReportDetail nrd = new NurseReportDetail();
      nrd.setAppointmentLocation(detail.getLocation());
      nrd.setAppointmentTime(Util.formatDateToDateTime(new Date(detail.getStartTimeStamp())));
      nrd.setClientDetails(per.getTitle() + " " + per.getFirstName() + " " + per.getSurname() + "\n"
          + detail.getPolicyNoAlpha() + detail.getPolicyNo());

      String reqs = "";
      for (int x = 0; x < ato[i].getClients().length; x++) {
        reqs += ato[i].getClients()[x].getPerson().getFirstName() + " "
            + ato[i].getClients()[x].getPerson().getSurname() + "\n";

        for (int j = 0; j < ato[i].getClients()[x].getRequirements().length; j++) {

          reqs += " -" + ato[i].getClients()[x].getRequirements()[j].getDescription() + "\n";
        }
      }
      // if (reqs.equals("")) {
      if (ato[i].getClients().length == 0) {
        reqs = detail.getComments();
      }

      // New fix - display additional requirements if no reqs available from db
      if (ato[i].getClients().length > 0) {
        if (ato[i].getClients()[0].getRequirements().length == 0) {
          reqs = ato[i].getExtraRequirements();
        }
      }
      nrd.setRequirements(reqs);
      nrd.setClientContactDetails("(c) " + con.getCellNo() + "\n" + "(w) " + con.getWorkPhoneNo() + "\n" + "(h) "
          + con.getHomePhoneNo());
      // Add new other number and additional details fields to outgoing contact details
      String newContactDetails = nrd.getClientContactDetails() + "\n" + "(other) " + con.getOtherPhoneNo() + "\n"
          + con.getSpecialInstructions();
      nrd.setClientContactDetails(newContactDetails);

      nurses.add(nrd);
    }
  }

  public String getContractRate() {
    return contractRate;
  }

  public void setContractRate(String contractRate) {
    this.contractRate = contractRate;
  }

  public void addNurseReportDetail(NurseReportDetail nurseDetail) {
    nurses.add(nurseDetail);
  }

  public void removeNurseReportDetail(NurseReportDetail nurseDetail) {
    nurses.remove(nurseDetail);
  }

  public NurseReportDetail[] getNurseReportDetail() {
    NurseReportDetail[] out = new NurseReportDetail[nurses.size()];
    int i = 0;
    for (Iterator iter = nurses.iterator(); iter.hasNext(); i++) {
      NurseReportDetail el = (NurseReportDetail) iter.next();
      out[i] = el;
    }
    return out;
  }

  public String getEndDate() {
    return endDate;
  }

  public void setEndDate(String endDate) {
    this.endDate = endDate;
  }

  public String getStartDate() {
    return startDate;
  }

  public void setStartDate(String startDate) {
    this.startDate = startDate;
  }

  public String getTheDate() {
    return theDate;
  }

  public void setTheDate(String theDate) {
    this.theDate = theDate;
  }

  public int getThreshold() {
    return threshold;
  }

  public void setThreshold(int threshold) {
    this.threshold = threshold;
  }

  public void addDCReportDetail(DCReportDetail dcDetail) {
    nurses.add(dcDetail);
  }

  public void removeDCReportDetail(DCReportDetail dcDetail) {
    nurses.remove(dcDetail);
  }

  public DCReportDetail[] getDCReportDetail() {
    DCReportDetail[] out = new DCReportDetail[nurses.size()];
    int i = 0;
    for (Iterator iter = nurses.iterator(); iter.hasNext(); i++) {
      DCReportDetail el = (DCReportDetail) iter.next();
      out[i] = el;
    }
    return out;
  }
}
