package za.co.momentum.recurring.nurse.client.reports;

import java.io.IOException;
import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.eclipse.core.runtime.Platform;
import za.co.momentum.recurring.nurse.util.TNLogger;

public class PermanentNursesReport extends ExcelReport {
	
	private double total = 0;
	
	public PermanentNursesReport(ExcelData data, String filename) {
		super(data, filename);
		try {
			templateName = Platform.asLocalURL(
					Platform.getProduct().getDefiningBundle().getEntry(
							"permanentNursesTemplate.xls")).getPath();
			TNLogger.debug("Template path = " + templateName);
		} catch (IOException e) {
			TNLogger.error("Could not find template permanentNursesTemplate.xls "
					+ e.getMessage());
		}
	}

	protected void writeReport() {
		HSSFCell c = getCell("$date");
		c.setCellValue(data.getTheDate());
		c = getCell("$startdate");
		c.setCellValue(data.getStartDate());
		c = getCell("$enddate");
		c.setCellValue(data.getEndDate());
		c = getCell("$rate");
		c.setCellValue(data.getContractRate());
		c = getCell("$threshold");
		c.setCellValue(data.getThreshold());
		c = getCell("$nurses");
		insertNurses(c);
		TNLogger.debug("Total is "+ total);
		c = getCell("$totalDue");
		c.setCellValue(total);
		TNLogger.debug("Finished writing report PermanentNursesReport");
	}

	private void insertNurses(HSSFCell startCell) {
		HSSFCell temp = startCell;
		HSSFRow row = sheet.getRow(rownum);
		NurseReportDetail[] detail = data.getNurseReportDetail();
		for (int i = 0; i < detail.length; i++) {
			temp = row.createCell((short) 0);
			temp.setCellValue(detail[i].getName());
			
			temp = row.createCell((short) (temp.getCellNum() + 1));
			temp.setCellValue(detail[i].getEmployeeNumber());
			
			temp = row.createCell((short) (temp.getCellNum() + 3));
			temp.setCellValue(detail[i].getNumberOfAppointmentDone());
			
			temp = row.createCell((short) (temp.getCellNum() + 2));
			temp.setCellValue(detail[i].getNumberOfAppointmentsAboveThreshold());
			
			temp = row.createCell((short) (temp.getCellNum() + 1));
			temp.setCellValue(detail[i].getSalary());
			
			total += detail[i].getSalary();
			row = sheet.createRow(row.getRowNum() + 1);
		}
	}
}
