/*
 * Momentum Risk & Savings IT
 * Project: Myriad
 * 
 * Created on 08 Mar 2012
 *
 */
package za.co.momentum.recurring.nurse.client.sao.eform;

import javax.xml.rpc.ServiceException;

import org.apache.log4j.Logger;

import za.co.momentum.recurring.nurse.client.model.ErrorMessages;
import za.co.momentum.recurring.nurse.client.sao.AbstractServiceManager;
import za.co.momentum.recurring.nurse.client.sao.ServiceAction;
import za.co.momentum.risk.fastlane.exceptions.EntityNotFoundException;
import za.co.momentum.risk.fastlane.nurse.dto.EFormTO;
import za.co.momentum.risk.fastlane.nurse.services.eform.EFormService;
import za.co.momentum.risk.fastlane.nurse.services.eform.EFormServiceRest;

public class EFormServiceManager extends AbstractServiceManager {
	/*
	 * A single instance of this class. It uses the singleton pattern.
	 */
	private static EFormServiceManager instance;
	/*
	 * This is the LOG instance.
	 */
	private static final Logger LOG = Logger
			.getLogger(EFormServiceManager.class);

	/**
	 * This is the hidden constructor so that we can make the class a
	 * singleton...
	 * 
	 */
	protected EFormServiceManager() {
	}

	/**
	 * Method for returning an instance of this class.
	 * 
	 * @return
	 */
	public static EFormServiceManager getInstance() {
		if (instance == null) {
			synchronized (EFormServiceManager.class) {
				if (instance == null) {
					instance = new EFormServiceManager();
				}
			}
		}
		return instance;
	}

	private EFormService getEFormImplementation() throws ServiceException {

		return EFormServiceRest.getInstance();

	}

	/**
	 * Method which will get all available E-Forms for a specific client id
	 * 
	 * @param username
	 * @param password
	 * @return
	 */
	public EFormTO[] getAvailableEFormsForClient(final long clientId) {
		// This is where the new implementation is located.
		Object returnObj = executeServiceAction(new ServiceAction() {
			public Object performService() throws Exception {
				return getEFormImplementation().getEFormsForClient(clientId);
			}
		});
		LOG.info("getAvailableEFormsForPolicy return object = " + returnObj);
		if (returnObj instanceof EFormTO[]) {
			// Got user now get roles on the user
			LOG.info("EForm returned:" + ((EFormTO[]) returnObj).length);
			// Return user
			return (EFormTO[]) returnObj;
		} else if (returnObj instanceof EntityNotFoundException) {
			ErrorMessages.showMessage(ErrorMessages.M23);
			return null;
		} else if (returnObj instanceof IllegalArgumentException) {
			ErrorMessages.showMessage(ErrorMessages.M0);
			return null;
		} else if (returnObj instanceof Exception) {
			return null;
		}
		return null;
	}

	public byte[] getEForm(final long eFormId) {
		// This is where the new implementation is located.
		Object returnObj = executeServiceAction(new ServiceAction() {
			public Object performService() throws Exception {
				return getEFormImplementation().getEform(eFormId);
			}
		});
		LOG.info("getEForm return object = " + returnObj);
		if (returnObj instanceof byte[]) {
			// Got user now get roles on the user
			LOG.info("Size of EForm image returned:"
					+ ((byte[]) returnObj).length);
			// Return user
			return (byte[]) returnObj;
		} else if (returnObj instanceof EntityNotFoundException) {
			ErrorMessages.showMessage(ErrorMessages.M23);
			return null;
		} else if (returnObj instanceof IllegalArgumentException) {
			ErrorMessages.showMessage(ErrorMessages.M0);
			return null;
		} else if (returnObj instanceof Exception) {
			return null;
		}
		return null;
	}

}
