package za.co.momentum.recurring.nurse.client.ui;

import java.io.IOException;
import org.apache.log4j.Logger;
import org.eclipse.core.runtime.preferences.ConfigurationScope;
import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.jface.preference.FileFieldEditor;
import org.eclipse.jface.preference.IntegerFieldEditor;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.eclipse.ui.preferences.ScopedPreferenceStore;
import za.co.momentum.recurring.client.TNPlugin.Application;

/**
 * This class represents a preference page that
 * is contributed to the Preferences dialog. By 
 * subclassing <samp>FieldEditorPreferencePage</samp>, we
 * can use the field support built into JFace that allows
 * us to create a page that is small and knows how to 
 * save, restore and apply itself.
 * <p>
 * This page is used to modify preferences only. They
 * are stored in the preference store that belongs to
 * the main plug-in class. That way, preferences can
 * be accessed directly via the preference store.
 */

public class PreferencePage
	extends FieldEditorPreferencePage
	implements IWorkbenchPreferencePage {

	private ScopedPreferenceStore preferences;
	private static final Logger LOG = Logger.getLogger(PreferencePage.class);
	
	public PreferencePage() {
		super(GRID);
		preferences = new ScopedPreferenceStore(new ConfigurationScope(), Application.PLUGIN_ID);
		setPreferenceStore(preferences);
		setDescription("System Preferences");
	}
	
	/**
	 * Creates the field editors. Field editors are abstractions of
	 * the common GUI blocks needed to manipulate various types
	 * of preferences. Each field editor knows how to save and
	 * restore itself.
	 */
	public void createFieldEditors() {
		addField(new IntegerFieldEditor(PreferenceConstants.TIME_OUT, 
				"&Server Time out:\n", getFieldEditorParent()));
//		addField(new IntegerFieldEditor(PreferenceConstants.START_HOUR, 
//				"&Calendar Start Hour:", getFieldEditorParent()));
//		addField(new IntegerFieldEditor(PreferenceConstants.END_HOUR, 
//				"&Calendar End Hour:", getFieldEditorParent()));
		addField(new FileFieldEditor(PreferenceConstants.PDF_EDITOR_PATH, 
				"&Acrobat Application Path and Executable:\n", getFieldEditorParent()));

	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchPreferencePage#init(org.eclipse.ui.IWorkbench)
	 */
	public void init(IWorkbench workbench) {
	}

	public boolean performOk() {
		try{
			preferences.save();
		}catch(IOException ex){
			LOG.debug("Could not save the preferences "+ ex.getMessage());
		}
		return super.performOk();
	}
	
}