package za.co.momentum.recurring.nurse.client.ui.wizards;

import java.math.BigDecimal;
import java.sql.Timestamp;
import java.util.Calendar;
import org.apache.log4j.Logger;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.SWT;
import org.eclipse.ui.PlatformUI;
import za.co.momentum.recurring.base.Util;
import za.co.momentum.recurring.base.email.EmailClient;
import za.co.momentum.recurring.nurse.client.model.ClientSecurityManager;
import za.co.momentum.recurring.nurse.client.sao.appreq.AppReqServiceManager;
import za.co.momentum.risk.fastlane.nurse.dto.AppDetailTO;
import za.co.momentum.risk.fastlane.nurse.dto.UserTO;
import za.co.momentum.recurring.nurse.util.ClientProperties;
import za.co.momentum.recurring.nurse.util.DBHelper;
import za.co.momentum.recurring.nurse.util.TNLogger;

import com.swtdesigner.SWTResourceManager;

public class RescheduleAppointmentWizzard {
	private AppDetailTO appreq;
	private Calendar date;
	private Calendar timeslot;
	private String appointmentForPerson;
	private int duration;
	private int prevDuration;
	private int returnCode = IDialogConstants.BACK_ID;

	public RescheduleAppointmentWizzard(AppDetailTO appreq) {
		this.appreq = appreq;
	}
	public int getReturnCode() {
		return returnCode;
	}
	public void run() {
		try {
			boolean end = false;
			int place = 0;
			while (!end) {
				if (place == 0) {
					RescheduleAppointmentWizzardPage1 page1 = new RescheduleAppointmentWizzardPage1(
							PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell());
					page1.open();
					date = page1.getSelectedDate();
					place += 1;
					returnCode = page1.getReturnCode();
					if (returnCode == IDialogConstants.CANCEL_ID) {
						end = true;
					}
				} else if (place == 1) {
					BigDecimal diff = Util.getTimestampDiffInSecs(new Timestamp(appreq
							.getEndTimeStamp()), new Timestamp(appreq.getStartTimeStamp()));
					diff = diff.divide(new BigDecimal(60), 0);
					prevDuration = diff.intValue();
					Logger.getLogger(RescheduleAppointmentWizzard.class).debug(
							"Duration of previous appointment was: " + prevDuration);
					RescheduleAppointmentWizzardPage2 page2 = new RescheduleAppointmentWizzardPage2(
							PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell(),
							date, prevDuration);
					page2.setUser(ClientSecurityManager.getInstance().getUser());
					page2.setNurse(appreq.getOriginator().getLastNurse());
					page2.setAppointmentForPerson(appreq.getContactPerson()
							.getFirstName()
							+ " " + appreq.getContactPerson().getSurname());
					page2.open();
					returnCode = page2.getReturnCode();
					timeslot = page2.getTimeslotSelected();
					duration = page2.getDuration();
					if (returnCode == IDialogConstants.BACK_ID) {
						place -= 1;
					} else if (returnCode == IDialogConstants.FINISH_ID) {
						// end
						place += 1;
						end = true;
					} else if (returnCode == IDialogConstants.CANCEL_ID) {
						end = true;
					}
				}
			}
			if (returnCode == IDialogConstants.FINISH_ID) {
				doFinish();
			}
		} catch (Exception ex) {
			ex.printStackTrace(System.out);
			sendErrorMail(ex);
		}
	}
	private static void sendErrorMail(Exception e) {
		String message = "Medperson RescheduleAppointmentWizzard: " + Util.getExceptionStackAsString(e);
		EmailClient.getInstance().sendMail(
				ClientProperties.getInstance().getProperty("email.fromAddress"),
				ClientProperties.getInstance().getProperty("email.techicalAddress"),
				"Reschedule Appointment Wizzard", message, null);
	}
	private void setValues() {
		appreq.setStartTimeStamp(timeslot.getTimeInMillis());
		Calendar temp = Calendar.getInstance();
		temp.setTimeInMillis(timeslot.getTimeInMillis());
		temp.add(Calendar.MINUTE, duration);
		appreq.setEndTimeStamp(temp.getTimeInMillis());
	}
	private void doFinish() {
    // First remove the appreq from the local db
    UserTO userTO = ClientSecurityManager.getInstance().getUser();
    UserTO adminUser = ClientSecurityManager.getInstance().getLoggedInUser();
    try {
    DBHelper.DeleteAppReq(appreq, userTO.getUsername());
    } catch (Exception e) {
      TNLogger.error("Error deleteing app from local db");
    }
    
		setValues();
		appreq.getOriginator().setUser(userTO);
		// appreq.set(Calendar.getInstance().getTimeInMillis());
		PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell().setCursor(SWTResourceManager.getCursor(SWT.CURSOR_WAIT));

        AppReqServiceManager.getInstance().unscheduleApp(userTO, adminUser, appreq);
		AppReqServiceManager.getInstance().bookAppointment(userTO, adminUser, appreq);
		
		PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell().setCursor(SWTResourceManager.getCursor(SWT.CURSOR_ARROW));
	}
}
