package za.co.momentum.recurring.nurse.db;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.URL;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Properties;
import org.eclipse.core.runtime.Platform;
import org.hsqldb.Server;
import org.hsqldb.persist.HsqlProperties;
import za.co.momentum.recurring.nurse.util.TNLogger;

public class DBManager {
	public static DBManager instance;
	private Server server;
	private Properties prop;

	private DBManager() {
		server = new Server();
		prop = new Properties();
		try {
			URL propFile = Platform.asLocalURL(Platform.getBundle("MedPersonPlugin")
					.getEntry("localNurse.properties"));
			prop.load(new FileInputStream(propFile.getFile()));
			HsqlProperties hprop = new HsqlProperties(prop);
			server.setProperties(hprop);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public static DBManager getinstance() {
		if (instance == null) {
			instance = new DBManager();
		}
		return instance;
	}

	public void startDB() {
		server.start();
		// Check if tables exists otherwise create them
		Connection con = null;
		try {
			con = ConnectionManager.getConnection();
			
      // TODO: detect if set
      PreparedStatement ps = con.prepareStatement("SET WRITE_DELAY FALSE");
      ps.execute();
      ps.close();
			
			createTable("EFORMDATA", "EFormTable.script", con);
			createTable("APPREQDATA", "AppReqTable.script", con);
			createTable("USERDATA", "UserTable.script", con);
			con.close();
		} catch (SQLException e) {
			TNLogger.debug("Error in creating dbs " + e.getMessage());
		} finally {
			ConnectionManager.cleanup(con);
		}
		TNLogger.debug("Finish with local db");
	}

	private void createTable(String tablename, String tableScript, Connection con) throws SQLException {
		TNLogger.debug("Now checking if " + tablename + " table exist");
		PreparedStatement st = con
				.prepareStatement("select * from INFORMATION_SCHEMA.SYSTEM_TABLES where TABLE_NAME=?");
		st.setString(1, tablename);
		ResultSet rs = st.executeQuery();
		boolean tableFound = false;
		while (rs.next()) {
			//TNLogger.debug("Adding a table");
			tableFound = true;
		}
		rs.close();
		st.close();
		// Check if table exists
		if (!tableFound) {
			TNLogger.debug("Now creating " + tablename + " table");
			String sql = prop.getProperty(tableScript);
			PreparedStatement ps = con.prepareStatement(sql);
			ps.executeUpdate();
			ps.close();
		}
	}

  public void stopDB() {
    try {
      Connection connection = ConnectionManager.getConnection();
      
      Statement statement = connection.createStatement();
      statement.execute("SHUTDOWN");
      statement.close();
      
      connection.close();
    } catch (SQLException e) {
    }
  }
}
