package za.co.momentum.recurring.nurse.util;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;
import java.util.List;

import za.co.momentum.recurring.nurse.client.model.User;
import za.co.momentum.recurring.nurse.db.ConnectionManager;
import za.co.momentum.risk.fastlane.nurse.dto.AppDetailTO;
import za.co.momentum.risk.fastlane.nurse.dto.UserTO;
import za.co.momentum.risk.fastlane.nurse.eforms.BaseForm;
import za.co.momentum.risk.fastlane.nurse.eforms.BasicPersonalDetailsForm;
import za.co.momentum.risk.fastlane.nurse.eforms.FinancialForm;
import za.co.momentum.risk.fastlane.nurse.eforms.ForeignInsuredForm;
import za.co.momentum.risk.fastlane.nurse.eforms.FormDTO;
import za.co.momentum.risk.fastlane.nurse.eforms.FullPersonalDetailsForm;
import za.co.momentum.risk.fastlane.nurse.eforms.NonSmokerForm;
import za.co.momentum.risk.fastlane.nurse.eforms.NurseDetailsForm;
import za.co.momentum.risk.fastlane.nurse.eforms.OutsideResidenceForm;
import za.co.momentum.risk.fastlane.nurse.eforms.PathologyRequestForm;
import za.co.momentum.risk.fastlane.nurse.eforms.ShortMedicalForm;

public class DBHelper {
	public static final String ShortMedical = "shortMedic";
	public static final String PathologyRequest = "patholoReq";
	public static final String OutsideResidence = "outsideRes";
	public static final String NurseDetails = "nursedetai";
	public static final String NonSmoker = "nonSmoker";
	public static final String FullPersonalDetails = "fullPDetai";
	public static final String ForeignInsured = "foreignIns";
	public static final String Financial = "financial";
	public static final String BasicPersonalDetails = "basicPDeta";
	public static final String Flattened = "flattened";
    public static final String FullData = "fulldata";

	public static BaseForm getForm(long refno, String formType) {
		TNLogger.debug("Entering getForm");
		BaseForm sm = null;
		byte[] ba = null;
		Connection con = null;
		try {
			con = ConnectionManager.getConnection();
			String sql = "select * from eformdata where refno=? and EFORMTYPE=?";
			PreparedStatement st = con.prepareStatement(sql);
			st.setLong(1, refno);
			st.setString(2, formType);
			ResultSet res = st.executeQuery();
			if (res.next()) {
				ba = res.getBytes("eformto");
			} else {
				return null;
			}
		} catch (SQLException e) {
			TNLogger.debug("SQL Exception " + e.getMessage());
			e.printStackTrace();
		} finally {
			ConnectionManager.cleanup(con);
		}
			sm = (BaseForm) Serializer.decode(ba);
		TNLogger.debug("Returning from getform " + sm);
		return sm;
	}
	
	public static AppDetailTO getAppReq(long refno, String username) {
		byte[] ba = null;
		Connection con = null;
		try {
			con = ConnectionManager.getConnection();
			String sql = "select * from appreqdata where refno=? and username=?";
			PreparedStatement st = con.prepareStatement(sql);
			st.setLong(1, refno);
			st.setString(2, username);
			ResultSet res = st.executeQuery();
			if (res.next()) {
				ba = res.getBytes("appreqto");
			} else {
				return null;
			}
		} catch (SQLException e) {
			e.printStackTrace();
		} finally {
			ConnectionManager.cleanup(con);
		}
		return (AppDetailTO) Serializer.deserialize(ba);
	}
	
	public static AppDetailTO[] getAllAppReqs(String username, int dayOfYear) {
		List<AppDetailTO> temp = new ArrayList<AppDetailTO>();
		byte[] ba = null;
		Connection con = null;
		try {
			con = ConnectionManager.getConnection();
			String sql = "select * from appreqdata where username=? and dayofyear=?";
			PreparedStatement st = con.prepareStatement(sql);
			st.setString(1, username);
			st.setInt(2, dayOfYear);
			ResultSet res = st.executeQuery();
			while (res.next()) {
				ba = res.getBytes("appreqto");
				temp.add((AppDetailTO) Serializer.deserialize(ba));
			}
		} catch (SQLException e) {
			e.printStackTrace();
		} finally {
			ConnectionManager.cleanup(con);
		}
		AppDetailTO[] sm = new AppDetailTO[temp.size()];
		int i = 0;
		for (Iterator<AppDetailTO> iter = temp.iterator(); iter.hasNext(); i++) {
			AppDetailTO el = (AppDetailTO) iter.next();
			sm[i] = el;
		}
		return sm;
	}
	
	public static boolean AddEForm(BaseForm form, long refno) {
//		byte[] ba = null;
		String formType = null;
		if (form instanceof ShortMedicalForm) {
			formType = ShortMedical;
		} else if (form instanceof PathologyRequestForm) {
			formType = PathologyRequest;
		} else if (form instanceof OutsideResidenceForm) {
			formType = OutsideResidence;
		} else if (form instanceof NurseDetailsForm) {
			formType = NurseDetails;
		} else if (form instanceof NonSmokerForm) {
			formType = NonSmoker;
		} else if (form instanceof FullPersonalDetailsForm) {
			formType = FullPersonalDetails;
		} else if (form instanceof ForeignInsuredForm) {
			formType = ForeignInsured;
		} else if (form instanceof FinancialForm) {
			formType = Financial;
		} else if (form instanceof BasicPersonalDetailsForm) {
          formType = BasicPersonalDetails;
    } else if (form instanceof FormDTO) {
      formType = FullData;
    }
		Connection con = null;
		try {
			con = ConnectionManager.getConnection();
			String sql = "update eformdata set eformto=? where EFORMTYPE=? and refno=?";
			// System.out.println(sql);
			PreparedStatement st = con.prepareStatement(sql);
			st.setLong(3, refno);
			st.setString(2, formType);
//			st.setBytes(1, ba);
			st.setBytes(1, Serializer.encode(form));
			int res = st.executeUpdate();
			if (res == 0) {
				sql = "insert into eformdata(refno, EFORMTYPE ,eformto) values(?,?,?)";
				// System.out.println(sql);
				st = con.prepareStatement(sql);
				st.setLong(1, refno);
				st.setString(2, formType);
//      st.setBytes(3, ba);
				st.setBytes(3, Serializer.encode(form));
				res = st.executeUpdate();
				if (res == 1) {
					return true;
				} else {
					return false;
				}
			} else {
				return true;
			}
		} catch (SQLException e) {
			e.printStackTrace();
		} finally {
			ConnectionManager.cleanup(con);
		}
		return false;
	}
	public static boolean AddAppReq(AppDetailTO appreq, String username) {
		byte[] ba = Serializer.serialize(appreq);
		
		Connection con = null;
		try {
			con = ConnectionManager.getConnection();
			String sql = "update appreqdata set appreqto=? where refno=? and username=?";
			// System.out.println(sql);
			PreparedStatement st = con.prepareStatement(sql);
			st.setLong(2, appreq.getId());
			st.setBytes(1, ba);
			st.setString(3, username);
			int res = st.executeUpdate();
			if (res == 0) {
				sql = "insert into appreqdata(refno,dayofyear,username,appreqto) values(?,?,?,?)";
				// System.out.println(sql);
				st = con.prepareStatement(sql);
				st.setLong(1, appreq.getId());
				Calendar c = Calendar.getInstance();
				c.setTimeInMillis(appreq.getStartTimeStamp());
				st.setInt(2, c.get(Calendar.DAY_OF_YEAR));
				st.setString(3, username);
				st.setBytes(4, ba);
				res = st.executeUpdate();
				if (res == 1) {
					return true;
				} else {
					return false;
				}
			} else {
				return true;
			}
		} catch (SQLException e) {
			e.printStackTrace();
		} finally {
			ConnectionManager.cleanup(con);
		}
		return false;
	}
	
	public static void DeleteAppReq(AppDetailTO appreq, String username) {
		Connection con = null;
		try {
			con = ConnectionManager.getConnection();
			String sql = "delete from appreqdata where refno=? and username=?";
			// System.out.println(sql);
			PreparedStatement st = con.prepareStatement(sql);
			st.setLong(1, appreq.getId());
			st.setString(2, username);
			st.executeUpdate();
		} catch (SQLException e) {
			// e.printStackTrace();
			TNLogger.error("DeleteAppreq failed to delete " + e.getMessage());
		} finally {
			ConnectionManager.cleanup(con);
		}
	}
	
	public static boolean addUser(UserTO userTo) {
		return addUser(new User(userTo));
	}
	
	public static boolean addUser(User user) {		
		byte[] ba = Serializer.serialize(user);
		
		Connection con = null;
		try {
			con = ConnectionManager.getConnection();
			String sql = "update userdata set userto=? where username=?";
			// System.out.println(sql);
			PreparedStatement st = con.prepareStatement(sql);
			st.setString(2, user.getUsername());
			st.setBytes(1, ba);
			int res = st.executeUpdate();
			if (res == 0) {
				sql = "insert into userdata(username,userto) values(?,?)";
				System.out.println(sql);
				st = con.prepareStatement(sql);
				st.setString(1, user.getUsername());
				st.setBytes(2, ba);
				res = st.executeUpdate();
				if (res == 1) {
					return true;
				} else {
					return false;
				}
			} else {
				return true;
			}
		} catch (SQLException e) {
			e.printStackTrace();
		} finally {
			ConnectionManager.cleanup(con);
		}
		return false;
	}
	
	public static User getUser(String username) {
		byte[] ba = null;
		Connection con = null;
		try {
			con = ConnectionManager.getConnection();
			String sql = "select * from userdata where username=? ";
			PreparedStatement st = con.prepareStatement(sql);
			st.setString(1, username);
			ResultSet res = st.executeQuery();
			if (res.next()) {
				ba = res.getBytes("userto");
			} else {
				return null;
			}
		} catch (SQLException e) {
			e.printStackTrace();
		} finally {
			ConnectionManager.cleanup(con);
		}		
		return (User) Serializer.deserialize(ba);
	}
	
	public static boolean clearAllUsers() {
		Connection con = null;
		try {
			con = ConnectionManager.getConnection();
			String sql = "delete from userdata";
			PreparedStatement st = con.prepareStatement(sql);
			st.executeUpdate();
		} catch (SQLException e) {
			e.printStackTrace();
		} finally {
			ConnectionManager.cleanup(con);
		}
		return false;
	}
}
