package za.co.momentum.recurring.nurse.util;

import java.util.Calendar;
import za.co.momentum.recurring.client.exceptions.TimeoutException;
import za.co.momentum.recurring.nurse.client.sao.ServiceAction;
import za.co.momentum.recurring.nurse.client.sao.WebThread;

public class WebCall {
	public static final int STATUS_QUEUED = 1;
	public static final int STATUS_BUSY = 2;
	public static final int STATUS_DONE = 3;
	public static final int STATUS_ERROR = 4;
	public static final int STATUS_TIMEOUT = 5;
	private int status;
	private Object returnObject;
	private ServiceAction theAction;
	private long refno = 0;
	// Retry of 0 means it will always retry
	private int retries = 0;
	private int retryCount = 0;
	private String error;
	private String description;

	

	public WebCall(ServiceAction action, long refno, int retries, String description) {
		theAction = action;
		this.refno = refno;
		this.retries = retries;
		this.description = description;
		error = "";
		status = STATUS_QUEUED;
	}
	
	public WebCall copy(WebCall from){
		WebCall out = new WebCall(from.getAction(), from.getRefno(), from.getRetries(), from.getDescription());
		out.retryCount = from.getRetryCount();
		out.error = from.getError();
		out.status = from.getStatus();
		return out;
	}
	
	public String getDescription(){
		return description;
	}
	
	public ServiceAction getAction(){
		return theAction;
	}
	
	public String getError() {
		return error;
	}

	public int getRetries() {
		return retries;
	}

	public void run() {
		returnObject = executeServiceAction();
	}

	private Object executeServiceAction() {
		//TNLogger.debug("Starting executeServiceAction of Webcall");
		retryCount++;
		status = STATUS_BUSY;
		returnObject = null;
		final long TIMEOUT = 60000;
		
		
		
		WebThread webCall = new WebThread(theAction);
		webCall.start();
		long endTS = System.currentTimeMillis() + TIMEOUT;
		long startTime = Calendar.getInstance().getTimeInMillis();
		while (webCall.isRunning() && System.currentTimeMillis() < endTS) {
			// Wait for return from webcall or timeout
			try {
				Thread.sleep(500);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
		returnObject = webCall.getReturnObject();
		
		if (returnObject instanceof TimeoutException) {
			status = STATUS_TIMEOUT;
			error = "Timeout";
			return null;
		}
//		TNLogger.info("WEBCALLER Web service profiling duration="
//				+ (Calendar.getInstance().getTimeInMillis() - startTime) + " ms");
//		TNLogger
//				.info("WEBCALLER Web service profiling object returned is "
//						+ returnObject);
		/*
		 * This error handling is AXIS specific. Must be changed if using another
		 * client web service implementation
		 */
//		if (returnObject instanceof AxisFault) {
//			status = STATUS_ERROR;
//			error = returnObject.toString();
//			return returnObject;
//		}
		TNLogger.debug("Return object = " + returnObject);
		status = STATUS_DONE;
		return returnObject;
	}

	public int getStatus() {
		return status;
	}

	public Object getReturnObject() {
		return returnObject;
	}

	public long getRefno() {
		return refno;
	}

	public int getRetryCount() {
		return retryCount;
	}
}
