package za.co.momentum.risk.fastlane.nurse.services.appreq;

import java.security.SecureRandom;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.List;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSession;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;
import javax.ws.rs.ProcessingException;
import javax.ws.rs.client.Client;
import javax.ws.rs.client.ClientBuilder;
import javax.ws.rs.client.Entity;
import javax.ws.rs.client.WebTarget;
import javax.ws.rs.core.Response;

import org.apache.log4j.Logger;
import org.glassfish.jersey.client.ClientConfig;
import org.glassfish.jersey.client.authentication.HttpAuthenticationFeature;

import za.co.momentum.recurring.exceptions.DomainRuntimeException;
import za.co.momentum.recurring.nurse.util.ClientProperties;
import za.co.momentum.recurring.nurse.util.ShieldUtils;
import za.co.momentum.risk.fastlane.exceptions.EntityNotFoundException;
import za.co.momentum.risk.fastlane.nurse.dto.AppDetailTO;
import za.co.momentum.risk.fastlane.nurse.dto.AppointmentStatusTO;
import za.co.momentum.risk.fastlane.nurse.dto.BranchTO;
import za.co.momentum.risk.fastlane.nurse.dto.OriginatorDetailsTO;
import za.co.momentum.risk.fastlane.nurse.dto.OriginatorLogTO;
import za.co.momentum.risk.fastlane.nurse.dto.PdfFormTO;
import za.co.momentum.risk.fastlane.nurse.dto.PersonTO;
import za.co.momentum.risk.fastlane.nurse.dto.PersonUpdatedDetailsTO;
import za.co.momentum.risk.fastlane.nurse.dto.ReportTO;
import za.co.momentum.risk.fastlane.nurse.dto.RoleTO;
import za.co.momentum.risk.fastlane.nurse.dto.SlimAppdetailTO;
import za.co.momentum.risk.fastlane.nurse.dto.TypeTO;
import za.co.momentum.risk.fastlane.nurse.dto.UserTO;
import za.co.momentum.risk.fastlane.nurse.exceptions.AppReqLockedException;
import za.co.momentum.risk.fastlane.nurse.exceptions.AppReqModifiedException;
import za.co.momentum.risk.fastlane.nurse.exceptions.CheckSumsNotEqualException;
import za.co.momentum.risk.fastlane.nurse.exceptions.TimeSlotNotAvailableException;

public class AppReqServiceRest implements AppReqService {

	private static final Logger LOG = Logger.getLogger(AppReqServiceRest.class);

	private WebTarget serviceTarget;
	private static AppReqServiceRest instance;

	public static AppReqServiceRest getInstance() {
		if (instance == null) {
			instance = new AppReqServiceRest();
		}
		return instance;
	}

	private AppReqServiceRest() {
		TrustManager[] certs = new TrustManager[] { new X509TrustManager() {
			@Override
			public X509Certificate[] getAcceptedIssuers() {
				return null;
			}

			@Override
			public void checkServerTrusted(X509Certificate[] chain,
					String authType) throws CertificateException {
			}

			@Override
			public void checkClientTrusted(X509Certificate[] chain,
					String authType) throws CertificateException {
			}
		} };
		SSLContext ctx = null;
		try {
			ctx = SSLContext.getInstance("TLSv1.2");
			ctx.init(null, certs, new SecureRandom());
		} catch (java.security.GeneralSecurityException e) {

		}
		HttpsURLConnection.setDefaultSSLSocketFactory(ctx.getSocketFactory());

		ClientBuilder clientBuilder = ClientBuilder.newBuilder();
		try {
			clientBuilder.sslContext(ctx);
			clientBuilder.hostnameVerifier(new HostnameVerifier() {

				@Override
				public boolean verify(String hostname, SSLSession session) {
					return true;
				}
			});
		} catch (Exception e) {
		}

		Client c = clientBuilder.withConfig(new ClientConfig()).build();
		String url = ClientProperties.getInstance().getProperty(
				"fastlane.appReqService");
		serviceTarget = c.target(url);
		ShieldUtils su = new ShieldUtils();
		serviceTarget.register(HttpAuthenticationFeature.basic(
				ClientProperties.getInstance().getProperty("fastlane.user"), 
			    su.translateKey(ClientProperties.getInstance().getProperty("fastlane.password"))));
	}

	@Override
	public SlimAppdetailTO[] getToBeScheduledAppReqs(String username)
			throws EntityNotFoundException {
		try {
			Response response = serviceTarget
					.path("/getToBeScheduledAppReqs/" + username).request()
					.get();
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(SlimAppdetailTO[].class);
			} else if (response.getStatus() == 404) {
				throw new EntityNotFoundException("Request:" + username);
			} else {
				String s = response.readEntity(String.class);
				LOG.error("Get getToBeScheduledAppReqs error. ResponseCode: "
						+ response.getStatus() + s);
				throw new DomainRuntimeException(
						"Error getToBeScheduledAppReqs:" + s);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		}
	}

	@Override
	public SlimAppdetailTO[] getPendingAppReqs(String username)
			throws EntityNotFoundException {
		try {
			Response response = serviceTarget
					.path("/getPendingAppReqs/" + username).request().get();
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(SlimAppdetailTO[].class);
			} else if (response.getStatus() == 404) {
				throw new EntityNotFoundException("Request:" + username);
			} else {
				String s = response.readEntity(String.class);
				LOG.error("Get getPendingAppReqs error. ResponseCode: "
						+ response.getStatus() + s);
				throw new DomainRuntimeException("Error getPendingAppReqs:" + s);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		}
	}

	@Override
	public SlimAppdetailTO[] getCompletedAppReqs(String username)
			throws EntityNotFoundException {
		try {
			Response response = serviceTarget
					.path("/getCompletedAppReqs/" + username).request().get();
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(SlimAppdetailTO[].class);
			} else if (response.getStatus() == 404) {
				throw new EntityNotFoundException("Request:" + username);
			} else {
				String s = response.readEntity(String.class);
				LOG.error("Get getCompletedAppReqs error. ResponseCode: "
						+ response.getStatus() + s);
				throw new DomainRuntimeException("Error getCompletedAppReqs:"
						+ s);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		}
	}

	@Override
	public void unlockAppReq(UserTO user, long originator) {
		Response response = null;
		try {
			UnlockAppReqRequest request = new UnlockAppReqRequest();
			request.setUser(user);
			request.setOriginator(originator);
			response = serviceTarget.path("unlockAppReq").request()
					.post(Entity.json(request));

			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				response.readEntity(String.class);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format("Error unlockAppReq: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}

	}

	@Override
	public void clearAllAppReqLocks() {
		Response response = null;
		try {
			response = serviceTarget.path("clearAllAppReqLocks").request()
					.post(Entity.json(""));

			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				response.readEntity(String.class);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format(
						"Error clearAllAppReqLocks: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public String appReqLockedByWhom(long originator) {
		try {
			Response response = serviceTarget
					.path("/appReqLockedByWhom/" + originator).request().get();
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(String.class);
			} else {
				String s = response.readEntity(String.class);
				LOG.error("Get appReqLockedByWhom error. ResponseCode: "
						+ response.getStatus() + s);
				throw new DomainRuntimeException("Error appReqLockedByWhom:"
						+ s);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		}
	}

	@Override
	public AppDetailTO getAppDetails(String username, long appreqId)
			throws EntityNotFoundException, AppReqLockedException {
		Response response = null;
		try {
			GetAppReqRequest request = new GetAppReqRequest();
			request.setUsername(username);
			request.setAppreqId(appreqId);
			response = serviceTarget.path("getAppDetails").request()
					.post(Entity.json(request));

			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(AppDetailTO.class);
			} else if (response.getStatus() == 404) {
				throw new EntityNotFoundException("Request:" + username);
			} else if (response.getStatus() == 403) {
				throw new AppReqLockedException("Request:" + username);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format("Error getAppDetails: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public SlimAppdetailTO[] searchAppReqs(String policyAlphaNumber,
			String policyNumber, String name, String surname) {
		Response response = null;
		try {
			SearchAppreqRequest request = new SearchAppreqRequest();
			request.setPolicyAlphaNumber(policyAlphaNumber);
			request.setPolicyNumber(policyNumber);
			request.setName(name);
			request.setSurname(surname);
			response = serviceTarget.path("searchAppReqs").request()
					.post(Entity.json(request));

			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(SlimAppdetailTO[].class);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format("Error searchAppReqs: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public AppDetailTO saveAppReq(UserTO user, UserTO adminUser,
			AppDetailTO appreq) throws AppReqModifiedException,
			EntityNotFoundException {
		Response response = null;
		try {
			SaveAppReqRequest request = new SaveAppReqRequest();
			request.setUser(user);
			request.setAdminUser(adminUser);
			request.setAppreq(appreq);
			response = serviceTarget.path("saveAppReq").request()
					.post(Entity.json(request));

			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(AppDetailTO.class);
			} else if (response.getStatus() == 404) {
				throw new EntityNotFoundException("Request:" + request);
			} else if (response.getStatus() == 403) {
				throw new AppReqModifiedException("Request:" + request);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format("Error saveAppReq: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public AppDetailTO setAdminStatsAction(UserTO user, AppDetailTO appDetail,
			long action) throws EntityNotFoundException {
		Response response = null;
		try {
			GenericRequest request = new GenericRequest();
			request.setUser(user);
			request.setAppDetail(appDetail);
			request.setAction(action);
			response = serviceTarget.path("setAdminStatsAction").request()
					.post(Entity.json(request));

			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(AppDetailTO.class);
			} else if (response.getStatus() == 404) {
				throw new EntityNotFoundException("Request:" + request);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format(
						"Error setAdminStatsAction: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public AppDetailTO setNoAnswerAction(UserTO user, AppDetailTO appDetail)
			throws EntityNotFoundException {
		Response response = null;
		try {
			GenericRequest request = new GenericRequest();
			request.setUser(user);
			request.setAppDetail(appDetail);
			response = serviceTarget.path("setNoAnswerAction").request()
					.post(Entity.json(request));

			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(AppDetailTO.class);
			} else if (response.getStatus() == 404) {
				throw new EntityNotFoundException("Request:" + request);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format("Error setNoAnswerAction: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public AppDetailTO setWrongNumberAction(UserTO user, AppDetailTO appDetail)
			throws EntityNotFoundException {
		Response response = null;
		try {
			GenericRequest request = new GenericRequest();
			request.setUser(user);
			request.setAppDetail(appDetail);
			response = serviceTarget.path("setWrongNumberAction").request()
					.post(Entity.json(request));

			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(AppDetailTO.class);
			} else if (response.getStatus() == 404) {
				throw new EntityNotFoundException("Request:" + request);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format(
						"Error setWrongNumberAction: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public AppDetailTO setClientDeclinedAction(UserTO user, UserTO adminUser,
			AppDetailTO appDetail) throws EntityNotFoundException {
		Response response = null;
		try {
			GenericRequest request = new GenericRequest();
			request.setUser(user);
			request.setAdminUser(adminUser);
			request.setAppDetail(appDetail);
			response = serviceTarget.path("setClientDeclinedAction").request()
					.post(Entity.json(request));

			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(AppDetailTO.class);
			} else if (response.getStatus() == 404) {
				throw new EntityNotFoundException("Request:" + request);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format(
						"Error setClientDeclinedAction: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public AppDetailTO setDiariseAction(UserTO user, AppDetailTO appDetail,
			long futureDate) throws EntityNotFoundException {
		Response response = null;
		try {
			GenericRequest request = new GenericRequest();
			request.setUser(user);
			request.setFutureDate(futureDate);
			request.setAppDetail(appDetail);
			response = serviceTarget.path("setDiariseAction").request()
					.post(Entity.json(request));

			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(AppDetailTO.class);
			} else if (response.getStatus() == 404) {
				throw new EntityNotFoundException("Request:" + request);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format("Error setDiariseAction: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public AppDetailTO setClientCancelledAction(UserTO user, UserTO adminUser,
			AppDetailTO appDetail) {
		Response response = null;
		try {
			GenericRequest request = new GenericRequest();
			request.setUser(user);
			request.setAdminUser(adminUser);
			request.setAppDetail(appDetail);
			response = serviceTarget.path("setClientCancelledAction").request()
					.post(Entity.json(request));

			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(AppDetailTO.class);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format(
						"Error setClientCancelledAction: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public AppDetailTO setNurseCancelledAction(UserTO user, UserTO adminUser,
			AppDetailTO appDetail) {
		Response response = null;
		try {
			GenericRequest request = new GenericRequest();
			request.setUser(user);
			request.setAdminUser(adminUser);
			request.setAppDetail(appDetail);
			response = serviceTarget.path("setNurseCancelledAction").request()
					.post(Entity.json(request));

			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(AppDetailTO.class);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format(
						"Error setNurseCancelledAction: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public String[] setAppointmentDoneAction(UserTO user, UserTO adminUser,
			AppDetailTO appDetail, PdfFormTO[] pdfFormTOS)
			throws EntityNotFoundException, CheckSumsNotEqualException {
		Response response = null;
		try {
			GenericRequest request = new GenericRequest();
			request.setUser(user);
			request.setAdminUser(adminUser);
			request.setAppDetail(appDetail);
			request.setPdfFormTOS(pdfFormTOS);
			response = serviceTarget.path("setAppointmentDoneAction").request()
					.post(Entity.json(request));

			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(String[].class);
			} else if (response.getStatus() == 404) {
				throw new EntityNotFoundException("Request:" + request);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format(
						"Error setAppointmentDoneAction: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public RoleTO[] getNursesBySuburb(String suburb)
			throws EntityNotFoundException {
		try {
			Response response = serviceTarget
					.path("/getNursesBySuburb/" + suburb).request().get();
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(RoleTO[].class);
			} else if (response.getStatus() == 404) {
				throw new EntityNotFoundException("Request:" + suburb);
			} else {
				String s = response.readEntity(String.class);
				LOG.error("Get getNursesBySuburb error. ResponseCode: "
						+ response.getStatus() + s);
				throw new DomainRuntimeException("Error getNursesBySuburb:" + s);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		}
	}

	@Override
	public AppDetailTO[] getAppReqsByNurseAndDate(long nurseId, long date)
			throws EntityNotFoundException {
		Response response = null;
		try {
			SearchAppreqRequest request = new SearchAppreqRequest();
			request.setNurseId(nurseId);
			request.setDate(date);
			response = serviceTarget.path("getAppReqsByNurseAndDate").request()
					.post(Entity.json(request));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(AppDetailTO[].class);
			} else if (response.getStatus() == 404) {
				throw new EntityNotFoundException("Request:" + request);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format(
						"Error getAppReqsByNurseAndDate: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public AppDetailTO[] getAppReqsByNurseAndDateFast(long nurseId, long date)
			throws EntityNotFoundException {
		Response response = null;
		try {
			SearchAppreqRequest request = new SearchAppreqRequest();
			request.setNurseId(nurseId);
			request.setDate(date);
			response = serviceTarget.path("getAppReqsByNurseAndDateFast")
					.request().post(Entity.json(request));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(AppDetailTO[].class);
			} else if (response.getStatus() == 404) {
				throw new EntityNotFoundException("Request:" + request);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format(
						"Error getAppReqsByNurseAndDateFast: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public AppDetailTO bookAppointment(UserTO user, UserTO adminUser,
			AppDetailTO appDetail) throws TimeSlotNotAvailableException {
		Response response = null;
		try {
			GenericRequest request = new GenericRequest();
			request.setUser(user);
			request.setAdminUser(adminUser);
			request.setAppDetail(appDetail);
			response = serviceTarget.path("bookAppointment").request()
					.post(Entity.json(request));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(AppDetailTO.class);
			} else if (response.getStatus() == 400) {
				throw new TimeSlotNotAvailableException("Request:" + request);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format("Error bookAppointment: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public AppDetailTO bookPersonalAppointment(UserTO user,
			AppDetailTO appDetail) throws TimeSlotNotAvailableException {
		Response response = null;
		try {
			GenericRequest request = new GenericRequest();
			request.setUser(user);
			request.setAppDetail(appDetail);
			response = serviceTarget.path("bookPersonalAppointment").request()
					.post(Entity.json(request));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(AppDetailTO.class);
			} else if (response.getStatus() == 400) {
				throw new TimeSlotNotAvailableException("Request:" + request);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format(
						"Error bookPersonalAppointment: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public AppDetailTO updatePersonalAppointment(UserTO user,
			AppDetailTO appDetail) throws TimeSlotNotAvailableException {
		Response response = null;
		try {
			GenericRequest request = new GenericRequest();
			request.setUser(user);
			request.setAppDetail(appDetail);
			response = serviceTarget.path("updatePersonalAppointment")
					.request().post(Entity.json(request));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(AppDetailTO.class);
			} else if (response.getStatus() == 400) {
				throw new TimeSlotNotAvailableException("Request:" + request);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format(
						"Error updatePersonalAppointment: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public boolean deletePersonalAppointment(UserTO user, AppDetailTO appDetail) {
		Response response = null;
		try {
			GenericRequest request = new GenericRequest();
			request.setUser(user);
			request.setAppDetail(appDetail);
			response = serviceTarget.path("deletePersonalAppointment")
					.request().post(Entity.json(request));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(Boolean.class);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format(
						"Error deletePersonalAppointment: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public OriginatorDetailsTO getAppReqById(UserTO user, long id)
			throws EntityNotFoundException, AppReqLockedException {
		Response response = null;
		try {
			SearchAppreqRequest request = new SearchAppreqRequest();
			request.setUser(user);
			request.setAppreqId(id);
			response = serviceTarget.path("getAppReqById").request()
					.post(Entity.json(request));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(OriginatorDetailsTO.class);
			} else if (response.getStatus() == 404) {
				throw new EntityNotFoundException("Request:" + id);
			} else if (response.getStatus() == 403) {
				throw new AppReqLockedException("Request:" + id);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format("Error getAppReqById: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public SlimAppdetailTO[] findClientsWithSamePolicyNum(long id) {
		try {
			Response response = serviceTarget
					.path("/findClientsWithSamePolicyNum/" + id).request()
					.get();
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(SlimAppdetailTO[].class);
			} else {
				String s = response.readEntity(String.class);
				LOG.error("Get findClientsWithSamePolicyNum error. ResponseCode: "
						+ response.getStatus() + s);
				throw new DomainRuntimeException(
						"Error findClientsWithSamePolicyNum:" + s);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		}
	}

	@Override
	public AppDetailTO setManualAppointment(UserTO user, AppDetailTO appDetail) {
		Response response = null;
		try {
			GenericRequest request = new GenericRequest();
			request.setUser(user);
			request.setAppDetail(appDetail);
			response = serviceTarget.path("setManualAppointment").request()
					.post(Entity.json(request));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(AppDetailTO.class);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format(
						"Error setManualAppointment: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public AppDetailTO[] getAppReqsForDate(long date) {
		try {
			Response response = serviceTarget
					.path("/getAppReqsForDate/" + date).request().get();
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(AppDetailTO[].class);
			} else {
				String s = response.readEntity(String.class);
				LOG.error("Get getAppReqsForDate error. ResponseCode: "
						+ response.getStatus() + s);
				throw new DomainRuntimeException("Error getAppReqsForDate:" + s);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		}
	}

	@Override
	public void closeAppReq(UserTO user, AppDetailTO appDetail)
			throws EntityNotFoundException {
		Response response = null;
		try {
			GenericRequest request = new GenericRequest();
			request.setUser(user);
			request.setAppDetail(appDetail);
			response = serviceTarget.path("closeAppReq").request()
					.post(Entity.json(request));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				response.readEntity(String.class);
			} else if (response.getStatus() == 404) {
				throw new EntityNotFoundException("Request:" + appDetail);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format("Error closeAppReq: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public AppDetailTO setUnScheduleAction(UserTO user, UserTO adminUser,
			AppDetailTO appDetail) throws EntityNotFoundException {
		Response response = null;
		try {
			GenericRequest request = new GenericRequest();
			request.setUser(user);
			request.setAdminUser(adminUser);
			request.setAppDetail(appDetail);
			response = serviceTarget.path("setUnScheduleAction").request()
					.post(Entity.json(request));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(AppDetailTO.class);
			} else if (response.getStatus() == 404) {
				throw new EntityNotFoundException("Request:" + appDetail);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format(
						"Error setUnScheduleAction: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public UserTO[] getUserWorkload() {
		try {
			Response response = serviceTarget.path("/getUserWorkload")
					.request().get();
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(UserTO[].class);
			} else {
				String s = response.readEntity(String.class);
				LOG.error("Get getUserWorkload error. ResponseCode: "
						+ response.getStatus() + s);
				throw new DomainRuntimeException("Error getUserWorkload:" + s);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		}
	}

	@Override
	public SlimAppdetailTO[] getWorkForUser(UserTO user) {
		Response response = null;
		try {
			GenericRequest request = new GenericRequest();
			request.setUser(user);
			response = serviceTarget.path("getWorkForUser").request()
					.post(Entity.json(request));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(SlimAppdetailTO[].class);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format("Error getWorkForUser: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public AppDetailTO switchUser(UserTO user, long appId, UserTO newUser) {
		Response response = null;
		try {
			GenericRequest request = new GenericRequest();
			request.setUser(user);
			request.setNewUser(newUser);
			request.setAppId(appId);
			response = serviceTarget.path("switchUser").request()
					.post(Entity.json(request));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(AppDetailTO.class);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format("Error switchUser: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public OriginatorLogTO[] getHistory(AppDetailTO appDetail)
			throws EntityNotFoundException {
		Response response = null;
		try {
			GenericRequest request = new GenericRequest();
			request.setAppDetail(appDetail);
			response = serviceTarget.path("getHistory").request()
					.post(Entity.json(request));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(OriginatorLogTO[].class);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format("Error getHistory: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public ReportTO[] getFastlaneStatus() {
		try {
			Response response = serviceTarget.path("/getFastlaneStatus")
					.request().get();
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(ReportTO[].class);
			} else {
				String s = response.readEntity(String.class);
				LOG.error("Get getFastlaneStatus error. ResponseCode: "
						+ response.getStatus() + s);
				throw new DomainRuntimeException("Error getFastlaneStatus:" + s);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		}
	}

	@Override
	public SlimAppdetailTO[] getAppsPerAppointmentStatus(
			AppointmentStatusTO status) {
		Response response = null;
		try {
			response = serviceTarget.path("getAppsPerAppointmentStatus")
					.request().post(Entity.json(status));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(SlimAppdetailTO[].class);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format(
						"Error getAppsPerAppointmentStatus: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public AppDetailTO setReOpenAction(UserTO user, AppDetailTO appDetail)
			throws EntityNotFoundException {
		Response response = null;
		try {
			GenericRequest request = new GenericRequest();
			request.setAppDetail(appDetail);
			request.setUser(user);
			response = serviceTarget.path("setReOpenAction").request()
					.post(Entity.json(request));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(AppDetailTO.class);
			} else if (response.getStatus() == 404) {
				throw new EntityNotFoundException("Request:" + appDetail);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format("Error setReOpenAction: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public AppDetailTO bookWorkRelatedAppointment(UserTO user, RoleTO nurse,
			AppDetailTO appDetail) throws TimeSlotNotAvailableException {
		Response response = null;
		try {
			GenericRequest request = new GenericRequest();
			request.setAppDetail(appDetail);
			request.setUser(user);
			request.setNurse(nurse);
			response = serviceTarget.path("bookWorkRelatedAppointment")
					.request().post(Entity.json(request));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(AppDetailTO.class);
			} else if (response.getStatus() == 400) {
				throw new TimeSlotNotAvailableException("Request:" + appDetail);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format(
						"Error bookWorkRelatedAppointment: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public void checkUnnecessaryDrBookings() {
		Response response = null;
		try {
			response = serviceTarget.path("checkUnnecessaryDrBookings")
					.request().post(Entity.json(""));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				response.readEntity(String.class);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format(
						"Error checkUnnecessaryDrBookings: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public void noMedPersonAvailable(UserTO user, AppDetailTO appDetail,
			String medPerson) throws EntityNotFoundException {
		Response response = null;
		try {
			GenericRequest request = new GenericRequest();
			request.setAppDetail(appDetail);
			request.setUser(user);
			request.setMedPerson(medPerson);
			response = serviceTarget.path("noMedPersonAvailable").request()
					.post(Entity.json(request));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				response.readEntity(String.class);
			} else if (response.getStatus() == 404) {
				throw new EntityNotFoundException("Request:" + medPerson);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format(
						"Error noMedPersonAvailable: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public AppDetailTO makeVitelCall(AppDetailTO appDetail, UserTO user,
			String telephoneNumber) {
		Response response = null;
		try {
			GenericRequest request = new GenericRequest();
			request.setAppDetail(appDetail);
			request.setUser(user);
			request.setTelephoneNumber(telephoneNumber);
			response = serviceTarget.path("makeVitelCall").request()
					.post(Entity.json(request));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(AppDetailTO.class);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format("Error makeVitelCall: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public void changeAppReqType(String policyAlpha, String policyNumber,
			String clientNumber, TypeTO typeTO) throws EntityNotFoundException {
		Response response = null;
		try {
			ChangeAppReqTypeRequest request = new ChangeAppReqTypeRequest();
			request.setPolicyAlpha(policyAlpha);
			request.setPolicyNumber(policyNumber);
			request.setClientNumber(clientNumber);
			request.setTypeTO(typeTO);
			response = serviceTarget.path("changeAppReqType").request()
					.post(Entity.json(request));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				response.readEntity(String.class);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format("Error changeAppReqType: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public void closeAppReqWithNoRequirements(UserTO user, AppDetailTO appDetail)
			throws EntityNotFoundException {
		Response response = null;
		try {
			GenericRequest request = new GenericRequest();
			request.setAppDetail(appDetail);
			request.setUser(user);
			response = serviceTarget.path("closeAppReqWithNoRequirements")
					.request().post(Entity.json(request));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				response.readEntity(String.class);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format(
						"Error closeAppReqWithNoRequirements: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public PersonTO[] findPersons(String clientNumber) {
		try {
			Response response = serviceTarget
					.path("/findPersons/" + clientNumber).request().get();
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(PersonTO[].class);
			} else {
				String s = response.readEntity(String.class);
				LOG.error("Get findPersons error. ResponseCode: "
						+ response.getStatus() + s);
				throw new DomainRuntimeException("Error findPersons:" + s);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		}
	}

	@Override
	public boolean updateAllPersons(String clientNumber,
			PersonUpdatedDetailsTO detailsTO) {
		Response response = null;
		try {
			UpdatePersonsRequest request = new UpdatePersonsRequest();
			request.setClientNumber(clientNumber);
			request.setDetailsTO(detailsTO);
			response = serviceTarget.path("updateAllPersons").request()
					.post(Entity.json(request));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(Boolean.class);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format("Error updateAllPersons: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public AppDetailTO setElectedOwnDoctorAction(UserTO user,
			AppDetailTO appDetail, String toAddress, String emailSubject,
			String emailMessage) throws EntityNotFoundException {
		Response response = null;
		try {
			GenericRequest request = new GenericRequest();
			request.setAppDetail(appDetail);
			request.setUser(user);
			request.setToAddress(toAddress);
			request.setEmailSubject(emailSubject);
			request.setEmailMessage(emailMessage);
			response = serviceTarget.path("setElectedOwnDoctorAction")
					.request().post(Entity.json(request));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(AppDetailTO.class);
			} else if (response.getStatus() == 404) {
				throw new EntityNotFoundException("Request:" + user);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format(
						"Error setElectedOwnDoctorAction: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public AppDetailTO assignToMedPerson(RoleTO initiator,
			AppDetailTO appDetailTO, RoleTO medPerson) {
		Response response = null;
		try {
			AssignToMedPersonRequest request = new AssignToMedPersonRequest();
			request.setAppDetailTO(appDetailTO);
			request.setInitiator(initiator);
			request.setMedPerson(medPerson);
			response = serviceTarget.path("assignToMedPerson").request()
					.post(Entity.json(request));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(AppDetailTO.class);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format("Error assignToMedPerson: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public BranchTO[] getAllBranches() {
		try {
			Response response = serviceTarget.path("/getAllBranches").request()
					.get();
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(BranchTO[].class);
			} else {
				String s = response.readEntity(String.class);
				LOG.error("Get getAllBranches error. ResponseCode: "
						+ response.getStatus() + s);
				throw new DomainRuntimeException("Error getAllBranches:" + s);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		}
	}

	@Override
	public Boolean saveBranch(BranchTO branch) {
		Response response = null;
		try {
			response = serviceTarget.path("saveBranch").request()
					.post(Entity.json(branch));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(Boolean.class);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format("Error saveBranch: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public Boolean deleteBranch(long id) {
		Response response = null;
		try {
			response = serviceTarget.path("deleteBranch").request()
					.post(Entity.json(id));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(Boolean.class);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format("Error deleteBranch: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public void sendGenericEmail(String fromAddress, String toAddress,
			String subject, String bodyText, Object[] filenames) {
		Response response = null;
		try {
			SendGenericEmailRequest request = new SendGenericEmailRequest();
			request.setFromAddress(fromAddress);
			request.setToAddress(toAddress);
			request.setSubject(subject);
			request.setBodyText(bodyText);
			List<String> files = new ArrayList<String>();
			for (int i = 0; i < filenames.length; i++) {
				files.add((String) filenames[i]);
			}
			request.setFileNames(files);
			response = serviceTarget.path("sendGenericEmail").request()
					.post(Entity.json(request));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				response.readEntity(String.class);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format("Error sendGenericEmail: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}

	}

	@Override
	public String saveNurseScheduleToSharedFolders(String filename) {
		Response response = null;
		try {
			response = serviceTarget.path("saveNurseScheduleToSharedFolders")
					.request().post(Entity.json(filename));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(String.class);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format(
						"Error saveNurseScheduleToSharedFolders: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

}
