package za.co.momentum.risk.fastlane.nurse.services.client;

import java.security.SecureRandom;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSession;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;
import javax.ws.rs.ProcessingException;
import javax.ws.rs.client.Client;
import javax.ws.rs.client.ClientBuilder;
import javax.ws.rs.client.Entity;
import javax.ws.rs.client.WebTarget;
import javax.ws.rs.core.Response;

import org.apache.log4j.Logger;
import org.glassfish.jersey.client.ClientConfig;
import org.glassfish.jersey.client.authentication.HttpAuthenticationFeature;

import za.co.momentum.recurring.exceptions.DomainRuntimeException;
import za.co.momentum.recurring.nurse.util.ClientProperties;
import za.co.momentum.recurring.nurse.util.ShieldUtils;
import za.co.momentum.risk.fastlane.nurse.dto.AppointmentStatusTO;
import za.co.momentum.risk.fastlane.nurse.dto.ClientAppointmentRequestDetailsTO;
import za.co.momentum.risk.fastlane.nurse.dto.ClientEFormTO;
import za.co.momentum.risk.fastlane.nurse.dto.ClientRequirementTO;
import za.co.momentum.risk.fastlane.nurse.dto.TypeTO;
import za.co.momentum.risk.fastlane.nurse.exceptions.InvalidAppointmentIdException;
import za.co.momentum.risk.fastlane.nurse.exceptions.InvalidBusinessSourceException;
import za.co.momentum.risk.fastlane.nurse.exceptions.InvalidDataException;
import za.co.momentum.risk.fastlane.nurse.exceptions.MissingDataException;

public class ClientServiceRest implements ClientService {

	private static final Logger LOG = Logger.getLogger(ClientServiceRest.class);

	private WebTarget serviceTarget;
	private static ClientServiceRest instance;

	public static ClientServiceRest getInstance() {
		if (instance == null) {
			instance = new ClientServiceRest();
		}
		return instance;
	}

	private ClientServiceRest() {
		TrustManager[] certs = new TrustManager[] { new X509TrustManager() {
			@Override
			public X509Certificate[] getAcceptedIssuers() {
				return null;
			}

			@Override
			public void checkServerTrusted(X509Certificate[] chain,
					String authType) throws CertificateException {
			}

			@Override
			public void checkClientTrusted(X509Certificate[] chain,
					String authType) throws CertificateException {
			}
		} };
		SSLContext ctx = null;
		try {
			ctx = SSLContext.getInstance("TLSv1.2");
			ctx.init(null, certs, new SecureRandom());
		} catch (java.security.GeneralSecurityException e) {

		}
		HttpsURLConnection.setDefaultSSLSocketFactory(ctx.getSocketFactory());

		ClientBuilder clientBuilder = ClientBuilder.newBuilder();
		try {
			clientBuilder.sslContext(ctx);
			clientBuilder.hostnameVerifier(new HostnameVerifier() {

				@Override
				public boolean verify(String hostname, SSLSession session) {
					return true;
				}
			});
		} catch (Exception e) {
		}

		Client c = clientBuilder.withConfig(new ClientConfig()).build();
		String url = ClientProperties.getInstance().getProperty(
				"fastlane.clientService");
		serviceTarget = c.target(url);
		ShieldUtils su = new ShieldUtils();
		serviceTarget.register(HttpAuthenticationFeature.basic(
				ClientProperties.getInstance().getProperty("fastlane.user"), 
			    su.translateKey(ClientProperties.getInstance().getProperty("fastlane.password"))));
	}

	@Override
	public long handleAppointment(ClientAppointmentRequestDetailsTO request)
			throws InvalidDataException, MissingDataException {
		Response response = null;
		try {
			response = serviceTarget.path("handleAppointment").request()
					.post(Entity.json(request));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(Long.class);
			} else if (response.getStatus() == 400) {
				String s = response.readEntity(String.class);
				if (s != null && s.contains("Missing")) {
					throw new MissingDataException("Request:" + request);
				} else {
					throw new InvalidDataException("Request:" + request);
				}
			} else {
				String s = response.readEntity(String.class);
				String error = String.format("Error handleAppointment: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public long[] getWorkDoneIds(TypeTO request)
			throws InvalidBusinessSourceException {
		Response response = null;
		try {
			response = serviceTarget.path("getWorkDoneIds").request()
					.post(Entity.json(request));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(long[].class);
			} else if (response.getStatus() == 400) {
				throw new InvalidBusinessSourceException("Request:" + request);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format("Error getWorkDoneIds: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public ClientAppointmentRequestDetailsTO getDoneAppointment(long id)
			throws InvalidAppointmentIdException {
		try {
			Response response = serviceTarget.path("/getDoneAppointment/" + id)
					.request().get();
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response
						.readEntity(ClientAppointmentRequestDetailsTO.class);
			} else if (response.getStatus() == 400) {
				throw new InvalidAppointmentIdException("Request:" + id);
			} else {
				String s = response.readEntity(String.class);
				LOG.error("Get getDoneAppointment error. ResponseCode: "
						+ response.getStatus() + s);
				throw new DomainRuntimeException("Error getDoneAppointment:"
						+ s);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		}
	}

	@Override
	public AppointmentStatusTO getAppointmentStatus(long id)
			throws InvalidAppointmentIdException {
		try {
			Response response = serviceTarget
					.path("/getAppointmentStatus/" + id).request().get();
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(AppointmentStatusTO.class);
			} else if (response.getStatus() == 400) {
				throw new InvalidAppointmentIdException("Request:" + id);
			} else {
				String s = response.readEntity(String.class);
				LOG.error("Get getAppointmentStatus error. ResponseCode: "
						+ response.getStatus() + s);
				throw new DomainRuntimeException("Error getAppointmentStatus:"
						+ s);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		}
	}

	@Override
	public ClientRequirementTO[] getAllAvailableMyriadRequirements() {
		try {
			Response response = serviceTarget
					.path("/getAllAvailableMyriadRequirements").request().get();
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(ClientRequirementTO[].class);
			} else {
				String s = response.readEntity(String.class);
				LOG.error("Get getAllAvailableMyriadRequirements error. ResponseCode: "
						+ response.getStatus() + s);
				throw new DomainRuntimeException(
						"Error getAllAvailableMyriadRequirements:" + s);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		}
	}

	@Override
	public boolean setAppReqFormsNotRetrieved(long id) {
		Response response = null;
		try {
			response = serviceTarget.path("setAppReqFormsNotRetrieved")
					.request().post(Entity.json(id));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(Boolean.class);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format(
						"Error setAppReqFormsNotRetrieved: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public boolean setEFormRetrievalFailed(long id) {
		Response response = null;
		try {
			response = serviceTarget.path("setEFormRetrievalFailed").request()
					.post(Entity.json(id));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(Boolean.class);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format(
						"Error setEFormRetrievalFailed: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public boolean setEFormRetrievalSuccess(long id) {
		Response response = null;
		try {
			response = serviceTarget.path("setEFormRetrievalSuccess").request()
					.post(Entity.json(id));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(Boolean.class);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format(
						"Error setEFormRetrievalSuccess: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public ClientEFormTO[] getRetrievalFailedEForms(TypeTO typeTO) {
		Response response = null;
		try {
			response = serviceTarget.path("getRetrievalFailedEForms").request()
					.post(Entity.json(typeTO));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(ClientEFormTO[].class);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format(
						"Error getRetrievalFailedEForms: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

}
