package za.co.momentum.risk.fastlane.nurse.services.eform;

import java.security.SecureRandom;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSession;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;
import javax.ws.rs.ProcessingException;
import javax.ws.rs.client.Client;
import javax.ws.rs.client.ClientBuilder;
import javax.ws.rs.client.Entity;
import javax.ws.rs.client.WebTarget;
import javax.ws.rs.core.Response;

import org.apache.log4j.Logger;
import org.glassfish.jersey.client.ClientConfig;
import org.glassfish.jersey.client.authentication.HttpAuthenticationFeature;

import za.co.momentum.recurring.exceptions.DomainRuntimeException;
import za.co.momentum.recurring.nurse.util.ClientProperties;
import za.co.momentum.recurring.nurse.util.ShieldUtils;
import za.co.momentum.risk.fastlane.nurse.dto.EFormTO;
import za.co.momentum.risk.fastlane.nurse.eforms.BasicPersonalDetailsForm;
import za.co.momentum.risk.fastlane.nurse.eforms.FinancialForm;
import za.co.momentum.risk.fastlane.nurse.eforms.ForeignInsuredForm;
import za.co.momentum.risk.fastlane.nurse.eforms.FormDTO;
import za.co.momentum.risk.fastlane.nurse.eforms.FullPersonalDetailsForm;
import za.co.momentum.risk.fastlane.nurse.eforms.NonSmokerForm;
import za.co.momentum.risk.fastlane.nurse.eforms.NurseDetailsForm;
import za.co.momentum.risk.fastlane.nurse.eforms.OutsideResidenceForm;
import za.co.momentum.risk.fastlane.nurse.eforms.PathologyRequestForm;
import za.co.momentum.risk.fastlane.nurse.eforms.ShortMedicalForm;

public class EFormServiceRest implements EFormService {

	private static final Logger LOG = Logger.getLogger(EFormServiceRest.class);

	private WebTarget serviceTarget;
	private static EFormServiceRest instance;

	public static EFormServiceRest getInstance() {
		if (instance == null) {
			instance = new EFormServiceRest();
		}
		return instance;
	}

	private EFormServiceRest() {
		TrustManager[] certs = new TrustManager[] { new X509TrustManager() {
			@Override
			public X509Certificate[] getAcceptedIssuers() {
				return null;
			}

			@Override
			public void checkServerTrusted(X509Certificate[] chain,
					String authType) throws CertificateException {
			}

			@Override
			public void checkClientTrusted(X509Certificate[] chain,
					String authType) throws CertificateException {
			}
		} };
		SSLContext ctx = null;
		try {
			ctx = SSLContext.getInstance("TLSv1.2");
			ctx.init(null, certs, new SecureRandom());
		} catch (java.security.GeneralSecurityException e) {

		}
		HttpsURLConnection.setDefaultSSLSocketFactory(ctx.getSocketFactory());

		ClientBuilder clientBuilder = ClientBuilder.newBuilder();
		try {
			clientBuilder.sslContext(ctx);
			clientBuilder.hostnameVerifier(new HostnameVerifier() {

				@Override
				public boolean verify(String hostname, SSLSession session) {
					return true;
				}
			});
		} catch (Exception e) {
		}

		Client c = clientBuilder.withConfig(new ClientConfig()).build();
		String url = ClientProperties.getInstance().getProperty(
				"fastlane.eFormService");
		serviceTarget = c.target(url);
		ShieldUtils su = new ShieldUtils();
		serviceTarget.register(HttpAuthenticationFeature.basic(
				ClientProperties.getInstance().getProperty("fastlane.user"), 
			    su.translateKey(ClientProperties.getInstance().getProperty("fastlane.password"))));
	}

	@Override
	public ShortMedicalForm prePopulateShortMedical(long id) {
		Response response = null;
		try {
			response = serviceTarget.path("prePopulateShortMedical").request()
					.post(Entity.json(id));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(ShortMedicalForm.class);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format(
						"Error prePopulateShortMedical: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public BasicPersonalDetailsForm prePopulateBasicPersonalDetails(long id) {
		Response response = null;
		try {
			response = serviceTarget.path("prePopulateBasicPersonalDetails")
					.request().post(Entity.json(id));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(BasicPersonalDetailsForm.class);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format(
						"Error prePopulateBasicPersonalDetails: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public FullPersonalDetailsForm prePopulateFullPersonalDetails(long id) {
		Response response = null;
		try {
			response = serviceTarget.path("prePopulateFullPersonalDetails")
					.request().post(Entity.json(id));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(FullPersonalDetailsForm.class);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format(
						"Error prePopulateFullPersonalDetails: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public NonSmokerForm prePopulateNonSmoker(long id) {
		Response response = null;
		try {
			response = serviceTarget.path("prePopulateNonSmoker").request()
					.post(Entity.json(id));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(NonSmokerForm.class);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format(
						"Error prePopulateNonSmoker: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public FinancialForm prePopulateFinancial(long id) {
		Response response = null;
		try {
			response = serviceTarget.path("prePopulateFinancial").request()
					.post(Entity.json(id));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(FinancialForm.class);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format(
						"Error prePopulateFinancial: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public OutsideResidenceForm prePopulateOutsideResidence(long id) {
		Response response = null;
		try {
			response = serviceTarget.path("prePopulateOutsideResidence")
					.request().post(Entity.json(id));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(OutsideResidenceForm.class);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format(
						"Error prePopulateOutsideResidence: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public ForeignInsuredForm prePopulateForeignInsured(long id) {
		Response response = null;
		try {
			response = serviceTarget.path("prePopulateForeignInsured")
					.request().post(Entity.json(id));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(ForeignInsuredForm.class);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format(
						"Error prePopulateForeignInsured: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public NurseDetailsForm prePopulateNurseDetails(long id) {
		Response response = null;
		try {
			response = serviceTarget.path("prePopulateNurseDetails").request()
					.post(Entity.json(id));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(NurseDetailsForm.class);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format(
						"Error prePopulateNurseDetails: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public PathologyRequestForm prePopulatePathReq(long id) {
		Response response = null;
		try {
			response = serviceTarget.path("prePopulatePathReq").request()
					.post(Entity.json(id));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(PathologyRequestForm.class);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format("Error prePopulatePathReq: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public void submitForm(byte[] bytes) {
		Response response = null;
		try {
			response = serviceTarget.path("submitForm").request()
					.post(Entity.json(bytes));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				response.readEntity(String.class);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format("Error submitForm: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public EFormTO[] getEFormsForClient(long clientNumber) {
		try {
			Response response = serviceTarget
					.path("/getEFormsForClient/" + clientNumber).request()
					.get();
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(EFormTO[].class);
			} else {
				String s = response.readEntity(String.class);
				LOG.error("Get getEFormsForClient error. ResponseCode: "
						+ response.getStatus() + s);
				throw new DomainRuntimeException("Error getEFormsForClient:"
						+ s);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		}
	}

	@Override
	public byte[] getEform(long id) {
		try {
			Response response = serviceTarget.path("/getEform/" + id).request()
					.get();
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(byte[].class);
			} else {
				String s = response.readEntity(String.class);
				LOG.error("Get getEform error. ResponseCode: "
						+ response.getStatus() + s);
				throw new DomainRuntimeException("Error getEform:" + s);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		}
	}

	@Override
	public boolean sendEFormToFilenet(long appReqId, long eFormId) {
		Response response = null;
		try {
			SendEFormToFilenetRequest request = new SendEFormToFilenetRequest();
			request.setAppReqId(appReqId);
			request.seteFormId(eFormId);
			response = serviceTarget.path("sendEFormToFilenet").request()
					.post(Entity.json(request));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(Boolean.class);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format("Error sendEFormToFilenet: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}

	}

	@Override
	public FormDTO getFormData(long id) {
		try {
			Response response = serviceTarget.path("/getFormData/" + id)
					.request().get();
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(FormDTO.class);
			} else {
				String s = response.readEntity(String.class);
				LOG.error("Get getFormData error. ResponseCode: "
						+ response.getStatus() + s);
				throw new DomainRuntimeException("Error getFormData:" + s);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		}
	}

}
