package za.co.momentum.risk.fastlane.nurse.services.reports;

import java.security.SecureRandom;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.Calendar;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSession;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;
import javax.ws.rs.ProcessingException;
import javax.ws.rs.client.Client;
import javax.ws.rs.client.ClientBuilder;
import javax.ws.rs.client.Entity;
import javax.ws.rs.client.WebTarget;
import javax.ws.rs.core.Response;

import org.apache.log4j.Logger;
import org.glassfish.jersey.client.ClientConfig;
import org.glassfish.jersey.client.authentication.HttpAuthenticationFeature;

import za.co.momentum.recurring.exceptions.DomainRuntimeException;
import za.co.momentum.recurring.nurse.util.ClientProperties;
import za.co.momentum.recurring.nurse.util.ShieldUtils;
import za.co.momentum.risk.fastlane.exceptions.EntityNotFoundException;
import za.co.momentum.risk.fastlane.nurse.dto.AppDetailTO;
import za.co.momentum.risk.fastlane.nurse.dto.AppsReportTO;
import za.co.momentum.risk.fastlane.nurse.dto.BranchReportTO;
import za.co.momentum.risk.fastlane.nurse.dto.ReportActionsCountDTO;
import za.co.momentum.risk.fastlane.nurse.dto.RoleTO;

public class ReportingServiceRest implements ReportingService {

	private static final Logger LOG = Logger
			.getLogger(ReportingServiceRest.class);

	private WebTarget serviceTarget;
	private static ReportingServiceRest instance;

	public static ReportingServiceRest getInstance() {
		if (instance == null) {
			instance = new ReportingServiceRest();
		}
		return instance;
	}

	private ReportingServiceRest() {
		TrustManager[] certs = new TrustManager[] { new X509TrustManager() {
			@Override
			public X509Certificate[] getAcceptedIssuers() {
				return null;
			}

			@Override
			public void checkServerTrusted(X509Certificate[] chain,
					String authType) throws CertificateException {
			}

			@Override
			public void checkClientTrusted(X509Certificate[] chain,
					String authType) throws CertificateException {
			}
		} };
		SSLContext ctx = null;
		try {
			ctx = SSLContext.getInstance("TLSv1.2");
			ctx.init(null, certs, new SecureRandom());
		} catch (java.security.GeneralSecurityException e) {

		}
		HttpsURLConnection.setDefaultSSLSocketFactory(ctx.getSocketFactory());

		ClientBuilder clientBuilder = ClientBuilder.newBuilder();
		try {
			clientBuilder.sslContext(ctx);
			clientBuilder.hostnameVerifier(new HostnameVerifier() {

				@Override
				public boolean verify(String hostname, SSLSession session) {
					return true;
				}
			});
		} catch (Exception e) {
		}

		Client c = clientBuilder.withConfig(new ClientConfig()).build();
		String url = ClientProperties.getInstance().getProperty(
				"fastlane.reportingService");
		serviceTarget = c.target(url);
		ShieldUtils su = new ShieldUtils();
		serviceTarget.register(HttpAuthenticationFeature.basic(
				ClientProperties.getInstance().getProperty("fastlane.user"), 
			    su.translateKey(ClientProperties.getInstance().getProperty("fastlane.password"))));
	}

	@Override
	public AppDetailTO[] getAppointmentsForTimePeriod(long from, long to,
			RoleTO nurse) throws EntityNotFoundException {
		Response response = null;
		try {
			ReportingRequest request = new ReportingRequest();
			request.setFrom(from);
			request.setTo(to);
			request.setNurse(nurse);
			response = serviceTarget.path("getAppointmentsForTimePeriod")
					.request().post(Entity.json(request));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(AppDetailTO[].class);
			} else if (response.getStatus() == 400) {
				throw new EntityNotFoundException("Request:" + request);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format(
						"Error getAppointmentsForTimePeriod: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public AppDetailTO[] getAppReqsForPeriodAndAction(long from, long to,
			long action, RoleTO nurse) throws EntityNotFoundException {
		Response response = null;
		try {
			ReportingRequest request = new ReportingRequest();
			request.setFrom(from);
			request.setTo(to);
			request.setNurse(nurse);
			request.setAction(action);
			response = serviceTarget.path("getAppReqsForPeriodAndAction")
					.request().post(Entity.json(request));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(AppDetailTO[].class);
			} else if (response.getStatus() == 400) {
				throw new EntityNotFoundException("Request:" + request);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format(
						"Error getAppReqsForPeriodAndAction: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public long getTimeInMinutesForAppreqToFirstAction(AppDetailTO appDetail) {
		Response response = null;
		try {
			response = serviceTarget
					.path("getTimeInMinutesForAppreqToFirstAction").request()
					.post(Entity.json(appDetail));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(Long.class);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format(
						"Error getTimeInMinutesForAppreqToFirstAction: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public long getTimeInMinutesForAppreqFromBookedToDone(AppDetailTO appDetail) {
		Response response = null;
		try {
			response = serviceTarget
					.path("getTimeInMinutesForAppreqFromBookedToDone")
					.request().post(Entity.json(appDetail));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(Long.class);
			} else {
				String s = response.readEntity(String.class);
				String error = String
						.format("Error getTimeInMinutesForAppreqFromBookedToDone: %s %s",
								response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public int getAppReqsBookedByDC(long from, long to, RoleTO dc, long action) {
		Response response = null;
		try {
			ReportingRequest request = new ReportingRequest();
			request.setFrom(from);
			request.setTo(to);
			request.setDc(dc);
			request.setAction(action);
			response = serviceTarget.path("getAppReqsBookedByDC").request()
					.post(Entity.json(request));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(Integer.class);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format(
						"Error getAppReqsBookedByDC: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public AppsReportTO[] getBookedDoneAppsByMedPerson(RoleTO[] roles,
			long from, long to) {
		Response response = null;
		try {
			ReportingRequest request = new ReportingRequest();
			request.setFrom(from);
			request.setTo(to);
			request.setRoles(roles);
			response = serviceTarget.path("getBookedDoneAppsByMedPerson")
					.request().post(Entity.json(request));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(AppsReportTO[].class);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format(
						"Error getBookedDoneAppsByMedPerson: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public AppsReportTO[] getBookedAppsByDC(RoleTO[] roles, long from, long to) {
		Response response = null;
		try {
			ReportingRequest request = new ReportingRequest();
			request.setFrom(from);
			request.setTo(to);
			request.setRoles(roles);
			response = serviceTarget.path("getBookedAppsByDC").request()
					.post(Entity.json(request));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(AppsReportTO[].class);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format("Error getBookedAppsByDC: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public BranchReportTO[] getBranchNewAppReqsForPeriod(long from, long to) {
		Response response = null;
		try {
			ReportingRequest request = new ReportingRequest();
			request.setFrom(from);
			request.setTo(to);
			response = serviceTarget.path("getBranchNewAppReqsForPeriod")
					.request().post(Entity.json(request));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(BranchReportTO[].class);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format(
						"Error getBranchNewAppReqsForPeriod: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public ReportActionsCountDTO[] getActionCountForAllDCs(Calendar startDate,
			Calendar endDate) {
		Response response = null;
		try {
			ReportingRequest request = new ReportingRequest();
			request.setStartDate(startDate);
			request.setEndDate(endDate);
			response = serviceTarget.path("getActionCountForAllDCs").request()
					.post(Entity.json(request));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(ReportActionsCountDTO[].class);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format(
						"Error getActionCountForAllDCs: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

}
